@@ -175,6 +175,15 @@ def annotation_to_nml(  # pylint: disable=dangerous-default-value
                 location=volume._default_zip_name(),
                 fallback_layer=volume.fallback_layer_name,
                 name=volume.name,
+                segments=[
+                    wknml.Segment(
+                        id=segment_id,
+                        name=segment_info.name,
+                        anchor_position=segment_info.anchor_position,
+                        color=segment_info.color,
+                    )
+                    for segment_id, segment_info in volume.segments.items()
+                ],
             )
         )
 
@@ -182,8 +191,8 @@ def annotation_to_nml(  # pylint: disable=dangerous-default-value
         wknml.Meta(name=key, content=value)
         for key, value in annotation.metadata.items()
     ]
-    if annotation.username is not None:
-        meta.append(wknml.Meta(name="username", content=annotation.username))
+    if annotation.owner_name is not None:
+        meta.append(wknml.Meta(name="username", content=annotation.owner_name))
     if annotation.annotation_id is not None:
         meta.append(wknml.Meta(name="annotationId", content=annotation.annotation_id))
 
