@@ -38,28 +38,40 @@ from typing import (
     Tuple,
     Union,
     cast,
+    overload,
 )
 from zipfile import ZIP_DEFLATED, ZipFile
 from zlib import Z_BEST_SPEED
 
 import attr
 import httpx
+from upath import UPath
 from zipp import Path as ZipPath
 
 import webknossos._nml as wknml
 from webknossos.annotation._nml_conversion import annotation_to_nml, nml_to_skeleton
+from webknossos.client._generated.api.default import dataset_info
 from webknossos.dataset import SEGMENTATION_CATEGORY, Dataset, Layer, SegmentationLayer
-from webknossos.geometry import BoundingBox
+from webknossos.dataset.dataset import RemoteDataset
+from webknossos.geometry import BoundingBox, Vec3Int
 from webknossos.skeleton import Skeleton
 from webknossos.utils import time_since_epoch_in_ms, warn_deprecated
 
 Vector3 = Tuple[float, float, float]
+Vector4 = Tuple[float, float, float, float]
 
 
 MAG_RE = r"((\d+-\d+-)?\d+)"
 SEP_RE = r"(\/|\\)"
-CUBE_RE = fr"z\d+{SEP_RE}y\d+{SEP_RE}x\d+\.wkw"
-ANNOTATION_WKW_PATH_RE = re.compile(fr"{MAG_RE}{SEP_RE}(header\.wkw|{CUBE_RE})")
+CUBE_RE = rf"z\d+{SEP_RE}y\d+{SEP_RE}x\d+\.wkw"
+ANNOTATION_WKW_PATH_RE = re.compile(rf"{MAG_RE}{SEP_RE}(header\.wkw|{CUBE_RE})")
+
+
+@attr.define
+class SegmentInformation:
+    name: Optional[str]
+    anchor_position: Optional[Vec3Int]
+    color: Optional[Vector4]
 
 
 @attr.define
@@ -68,6 +80,7 @@ class _VolumeLayer:
     name: str
     fallback_layer_name: Optional[str]
     zip: Optional[ZipPath]
+    segments: Dict[int, SegmentInformation]
 
     def _default_zip_name(self) -> str:
         return f"data_{self.id}_{self.name}.zip"
@@ -83,7 +96,7 @@ class Annotation:
     _voxel_size: Optional[Vector3] = None
     _organization_id: Optional[str] = None
     _description: Optional[str] = None
-    username: Optional[str] = None
+    owner_name: Optional[str] = None
     annotation_id: Optional[str] = None
     time: Optional[int] = attr.ib(factory=time_since_epoch_in_ms)
     edit_position: Optional[Vector3] = None
@@ -135,6 +148,17 @@ class Annotation:
                 + "The attributes of the skeleton are used in this case."
             )
 
+    @property
+    def username(self) -> Optional[str]:
+        """Deprecated, use `owner_name` instead."""
+        warn_deprecated("username", "owner_name")
+        return self.owner_name
+
+    @username.setter
+    def username(self, username: str) -> None:
+        warn_deprecated("username", "owner_name")
+        self.owner_name = username
+
     @property
     def dataset_name(self) -> str:
         """This attribute is a proxy for `skeleton.dataset_name`."""
@@ -195,33 +219,57 @@ class Annotation:
             return cls._load_from_zip(annotation_path)
         elif annotation_path.suffix == ".nml":
             with annotation_path.open(mode="rb") as f:
-                annotation, nml = cls._load_from_nml(annotation_path.stem, f)
-            if len(nml.volumes) > 0:
-                warnings.warn(
-                    "The loaded nml contains references to volume layer annotations. "
-                    + "Those can only be loaded from a zip-file containing the nml and the volume annotation layer zips. "
-                    + "Omitting the volume layer annotations."
-                )
-            return annotation
+                return cls._load_from_nml(annotation_path.stem, f)
         else:
             raise RuntimeError(
                 "The loaded annotation must have the suffix .zip or .nml, but is {annotation_path.suffix}"
             )
 
+    @overload
+    @classmethod
+    def download(
+        cls,
+        annotation_id_or_url: str,
+        annotation_type: Union[str, "AnnotationType", None] = None,
+        webknossos_url: Optional[str] = None,
+        *,
+        skip_volume_data: bool = False,
+    ) -> "Annotation":
+        ...
+
+    @overload
+    @classmethod
+    def download(
+        cls,
+        annotation_id_or_url: str,
+        annotation_type: Union[str, "AnnotationType", None] = None,
+        webknossos_url: Optional[str] = None,
+        *,
+        skip_volume_data: bool = False,
+        _return_context: bool,
+    ) -> Tuple["Annotation", ContextManager[None]]:
+        ...
+
     @classmethod
     def download(
         cls,
         annotation_id_or_url: str,
         annotation_type: Union[str, "AnnotationType", None] = None,
         webknossos_url: Optional[str] = None,
-    ) -> "Annotation":
+        *,
+        skip_volume_data: bool = False,
+        _return_context: bool = False,
+    ) -> Union["Annotation", Tuple["Annotation", ContextManager[None]]]:
         """
         * `annotation_id_or_url` may be an annotation id or a full URL to an annotation, e.g.
-          `https://webknossos.org/annotations/Explorational/6114d9410100009f0096c640`
-        * `annotation_type` must be supplied if and only if an annotation id was used in the previous argument
+          `https://webknossos.org/annotations/6114d9410100009f0096c640`
+        * `annotation_type` is no longer required and therefore deprecated and ignored
         * `webknossos_url` may be supplied if an annotation id was used
           and allows to specifiy in which webknossos instance to search for the annotation.
           It defaults to the url from your current `webknossos_context`, using https://webknossos.org as a fallback.
+        * `skip_volume_data` can be set to `True` to omit downloading annotated volume data.
+          They can still be streamed from webKnossos using `annotation.get_remote_annotation_dataset()`.
+        * `_return_context` should not be set.
         """
         from webknossos.client._generated.api.default import annotation_download
         from webknossos.client.context import (
@@ -234,23 +282,19 @@ class Annotation:
         if match is not None:
             assert webknossos_url is None and annotation_type is None, (
                 "When Annotation.download() is be called with an annotation url, "
-                + "e.g. Annotation.download('https://webknossos.org/annotations/Explorational/6114d9410100009f0096c640'), "
+                + "e.g. Annotation.download('https://webknossos.org/annotations/6114d9410100009f0096c640'), "
                 + "annotation_type and webknossos_url must not be set."
             )
             annotation_id = match.group("annotation_id")
-            annotation_type = match.group("annotation_type")
             webknossos_url = match.group("webknossos_url")
         else:
-            assert annotation_type is not None, (
-                "When calling Annotation.download() with an id you must supply the argument annotation_type, "
-                + "e.g. 'Task' or 'Explorational'. Alternatively, you can use the full annotation url, "
-                + "e.g. Annotation.download('https://webknossos.org/annotations/Explorational/6114d9410100009f0096c640')."
-            )
             annotation_id = annotation_id_or_url
-        annotation_type = AnnotationType(annotation_type)
-        assert (
-            annotation_type not in _COMPOUND_ANNOTATION_TYPES
-        ), f"Currently compound annotation types are not supported, got {annotation_type}"
+
+        if annotation_type is not None:
+            warnings.warn(
+                "[DEPRECATION] `annotation_type` is deprecated for Annotation.download(), it should be omitted.",
+                DeprecationWarning,
+            )
 
         if webknossos_url is not None and webknossos_url != _get_context().url:
             warnings.warn(
@@ -267,52 +311,146 @@ class Annotation:
         with context:
             client = _get_generated_client()
             response = annotation_download.sync_detailed(
-                typ=annotation_type.value, id=annotation_id, client=client
+                id=annotation_id,
+                client=client,
+                skip_volume_data=skip_volume_data,
             )
         assert response.status_code == 200, response
         content_disposition_header = response.headers.get("content-disposition", "")
         _header_value, header_params = cgi.parse_header(content_disposition_header)
         filename = header_params.get("filename", "")
         if filename.endswith(".nml"):
-            annotation, nml = Annotation._load_from_nml(
+            annotation = Annotation._load_from_nml(
                 filename[:-4], BytesIO(response.content)
             )
-            assert (
-                len(nml.volumes) == 0
-            ), "The downloaded NML contains volume tags, it should have downloaded a zip instead."
-            return annotation
         else:
             assert filename.endswith(
                 ".zip"
             ), f"Downloaded annoation should have the suffix .zip or .nml, but has filename {filename}"
-            return Annotation._load_from_zip(BytesIO(response.content))
+            annotation = Annotation._load_from_zip(BytesIO(response.content))
+
+        if _return_context:
+            return annotation, context
+        else:
+            return annotation
+
+    @classmethod
+    def open_as_remote_dataset(
+        cls,
+        annotation_id_or_url: str,
+        annotation_type: Union[str, "AnnotationType", None] = None,
+        webknossos_url: Optional[str] = None,
+    ) -> Dataset:
+        (  # pylint: disable=unpacking-non-sequence
+            annotation,
+            context,
+        ) = Annotation.download(
+            annotation_id_or_url=annotation_id_or_url,
+            annotation_type=annotation_type,
+            webknossos_url=webknossos_url,
+            skip_volume_data=True,
+            _return_context=True,
+        )
+
+        with context:
+            return annotation.get_remote_annotation_dataset()
 
     @classmethod
     def _load_from_nml(
-        cls, name: str, nml_content: BinaryIO
-    ) -> Tuple["Annotation", wknml.Nml]:
+        cls,
+        name: str,
+        nml_content: BinaryIO,
+        possible_volume_paths: Optional[List[ZipPath]] = None,
+    ) -> "Annotation":
         nml = wknml.Nml.parse(nml_content)
 
-        return (
-            cls(
-                name=name,
-                skeleton=nml_to_skeleton(nml),
-                username=nml.get_meta("username"),
-                annotation_id=nml.get_meta("annotationId"),
-                time=nml.parameters.time,
-                edit_position=nml.parameters.editPosition,
-                edit_rotation=nml.parameters.editRotation,
-                zoom_level=nml.parameters.zoomLevel,
-                task_bounding_box=nml.parameters.taskBoundingBox,
-                user_bounding_boxes=nml.parameters.userBoundingBoxes or [],
-                metadata={
-                    i.name: i.content
-                    for i in nml.meta
-                    if i.name not in ["username", "annotationId"]
-                },
-            ),
-            nml,
+        annotation = cls(
+            name=name,
+            skeleton=nml_to_skeleton(nml),
+            owner_name=nml.get_meta("username"),
+            annotation_id=nml.get_meta("annotationId"),
+            time=nml.parameters.time,
+            edit_position=nml.parameters.editPosition,
+            edit_rotation=nml.parameters.editRotation,
+            zoom_level=nml.parameters.zoomLevel,
+            task_bounding_box=nml.parameters.taskBoundingBox,
+            user_bounding_boxes=nml.parameters.userBoundingBoxes or [],
+            metadata={
+                i.name: i.content
+                for i in nml.meta
+                if i.name not in ["username", "annotationId"]
+            },
         )
+        annotation._volume_layers = cls._parse_volumes(nml, possible_volume_paths)
+        return annotation
+
+    @staticmethod
+    def _parse_volumes(
+        nml: wknml.Nml, possible_paths: Optional[List[ZipPath]]
+    ) -> List[_VolumeLayer]:
+        volume_layers = []
+        layers_with_not_found_location = []
+        layers_without_location = []
+        for volume in nml.volumes:
+            if possible_paths is None:  # when parsing NML files
+                volume_path = None
+                if volume.location is not None:
+                    # This should only happen if a zipped nml
+                    # is unpacked and loaded directly.
+                    layers_with_not_found_location.append(volume)
+                    volume_path = None
+            elif volume.location is None:
+                volume_path = None
+                layers_without_location.append(volume)
+            else:
+                fitting_volume_paths = [
+                    i for i in possible_paths if str(i.at) == volume.location
+                ]
+                if len(fitting_volume_paths) == 1:
+                    with fitting_volume_paths[0].open(mode="rb") as f:
+                        with ZipFile(f) as volume_layer_zipfile:
+                            if len(volume_layer_zipfile.filelist) == 0:
+                                volume_path = None
+                            else:
+                                volume_path = fitting_volume_paths[0]
+                else:
+                    layers_with_not_found_location.append(volume)
+                    volume_path = None
+
+            segments = {}
+            if volume.segments is not None:
+                for segment in volume.segments:
+                    segments[segment.id] = SegmentInformation(
+                        name=segment.name,
+                        anchor_position=segment.anchor_position,
+                        color=segment.color,
+                    )
+            volume_layers.append(
+                _VolumeLayer(
+                    id=volume.id,
+                    name="Volume" if volume.name is None else volume.name,
+                    fallback_layer_name=volume.fallback_layer,
+                    zip=volume_path,
+                    segments=segments,
+                )
+            )
+        assert len(set(i.id for i in volume_layers)) == len(
+            volume_layers
+        ), "Some volume layers have the same id, this is not allowed."
+        if len(layers_without_location) > 0:
+            warnings.warn(
+                "Omitting the volume layer annotation data for layers "
+                + f"{[v.name or v.id for v in layers_without_location]}, "
+                + "as their location is not referenced in the NML."
+            )
+        if len(layers_with_not_found_location) > 0:
+            warnings.warn(
+                "Omitting the volume layer annotation data for layers "
+                + f"{[v.name or v.id for v in layers_without_location]}, "
+                + f"as their referenced files {[v.location for v in layers_without_location]} "
+                + "cannot be found."
+            )
+        return volume_layers
 
     @classmethod
     def _load_from_zip(cls, content: Union[str, PathLike, BinaryIO]) -> "Annotation":
@@ -324,32 +462,7 @@ class Annotation:
             len(nml_paths) == 1
         ), f"There must be exactly one nml file in the zip-file, buf found {len(nml_paths)}."
         with nml_paths[0].open(mode="rb") as f:
-            annotation, nml = cls._load_from_nml(nml_paths[0].stem, f)
-        volume_layers = []
-        for volume in nml.volumes:
-            fitting_volume_paths = [i for i in paths if str(i.at) == volume.location]
-            assert (
-                len(fitting_volume_paths) == 1
-            ), f"Couldn't find the file {volume.location} for the volume annotation {volume.name or volume.id}"
-            with fitting_volume_paths[0].open(mode="rb") as f:
-                with ZipFile(f) as volume_layer_zipfile:
-                    if len(volume_layer_zipfile.filelist) == 0:
-                        volume_path = None
-                    else:
-                        volume_path = fitting_volume_paths[0]
-            volume_layers.append(
-                _VolumeLayer(
-                    id=volume.id,
-                    name="Volume" if volume.name is None else volume.name,
-                    fallback_layer_name=volume.fallback_layer,
-                    zip=volume_path,
-                )
-            )
-        assert len(set(i.id for i in volume_layers)) == len(
-            volume_layers
-        ), "Some volume layers have the same id, this is not allowed."
-        annotation._volume_layers = volume_layers
-        return annotation
+            return cls._load_from_nml(nml_paths[0].stem, f, possible_volume_paths=paths)
 
     def save(self, path: Union[str, PathLike]) -> None:
         """
@@ -426,6 +539,69 @@ class Annotation:
                 layer_content,
             )
 
+    def get_remote_annotation_dataset(self) -> Dataset:
+        """
+        Returns a streamed dataset of the annotation as shown in webknossos,
+        incorporating fallback layers and potentially mappings.
+        A mapping is currently only incorporated if it is a pinned agglomerate mapping.
+        After an agglomerate mapping was activated in webKnossos, it is pinned as soon
+        as the first volume editing action is done. Note that this behavior might change
+        in the future.
+        """
+        from webknossos.client.context import _get_context
+
+        if self.annotation_id is None:
+            raise ValueError(
+                "The annotation_id is not set, cannot get the corresponding dataset."
+            )
+
+        context = _get_context()
+        token: Optional[str]
+        if self.organization_id is None:
+            token = context.required_token
+            organization_id = context.organization_id
+        else:
+            token = context.token
+            organization_id = self.organization_id
+            # avoid requiring authentication
+            if token is not None:
+                if organization_id != context.organization_id:
+                    warnings.warn(
+                        "The annotation used with get_remote_annotation_dataset "
+                        + "specifies a different organization id than the current context. "
+                        + f"The annotation uses {organization_id}, the context {context.organization_id}.",
+                        RuntimeWarning,
+                    )
+
+        dataset_info_response = dataset_info.sync_detailed(
+            organization_name=organization_id,
+            data_set_name=self.dataset_name,
+            client=context.generated_client,
+        )
+        assert dataset_info_response.status_code == 200, dataset_info_response
+        parsed = dataset_info_response.parsed
+        assert parsed is not None
+
+        datastore_url = parsed.data_store.url
+
+        zarr_path = UPath(
+            f"{datastore_url}/data/annotations/zarr/{self.annotation_id}/",
+            headers={} if token is None else {"X-Auth-Token": token},
+        )
+        return Dataset.open(zarr_path)
+
+    def get_remote_base_dataset(
+        self,
+        sharing_token: Optional[str] = None,
+        webknossos_url: Optional[str] = None,
+    ) -> RemoteDataset:
+        return Dataset.open_remote(
+            self.dataset_name,
+            self.organization_id,
+            sharing_token=sharing_token,
+            webknossos_url=webknossos_url,
+        )
+
     def get_volume_layer_names(self) -> Iterable[str]:
         return (i.name for i in self._volume_layers)
 
@@ -465,6 +641,7 @@ class Annotation:
                 name=name,
                 fallback_layer_name=fallback_layer_name,
                 zip=None,
+                segments={},
             )
         )
 
@@ -565,7 +742,7 @@ class Annotation:
 
         assert (
             volume_zip_path is not None
-        ), "The selected volume layer is empty and cannot be exported."
+        ), "The selected volume layer data is not available and cannot be exported."
 
         with volume_zip_path.open(mode="rb") as f:
             data_zip = ZipFile(f)
@@ -588,8 +765,11 @@ class Annotation:
 
         if largest_segment_id is None:
             max_value = max(
-                view.read().max()
-                for view in best_mag_view.get_views_on_disk(read_only=True)
+                (
+                    view.read().max()
+                    for view in best_mag_view.get_views_on_disk(read_only=True)
+                ),
+                default=0,
             )
             layer.largest_segment_id = int(max_value)
         else:
@@ -626,8 +806,25 @@ class Annotation:
                 volume_layer_id=volume_layer_id,
             )
 
+            input_annotation_dataset._read_only = True
+
             yield input_annotation_layer
 
+    def get_volume_layer_segments(
+        self,
+        volume_layer_name: Optional[str] = None,
+        volume_layer_id: Optional[int] = None,
+    ) -> Dict[int, SegmentInformation]:
+        """Returns a dict mapping from segment ids to `SegmentInformation`.
+        The dict is mutable, changes to the returned instance are saved in the local annotation.
+        Changes in a downloaded annotation that are done online in webknossos are not
+        reflected automatically, the annotation needs to be re-downloaded."""
+        layer = self._get_volume_layer(
+            volume_layer_name=volume_layer_name,
+            volume_layer_id=volume_layer_id,
+        )
+        return layer.segments
+
 
 Annotation._set_init_docstring()
 
@@ -674,7 +871,7 @@ _COMPOUND_ANNOTATION_TYPES = [
 
 _ANNOTATION_URL_REGEX = re.compile(
     r"^(?P<webknossos_url>https?://.*)/annotations/"
-    + fr"(?P<annotation_type>{'|'.join(i.value for i in AnnotationType.__members__.values())})/"
+    + rf"((?P<annotation_type>{'|'.join(i.value for i in AnnotationType.__members__.values())})/)?"
     + r"(?P<annotation_id>[0-9A-Fa-f]*)"
 )
 
