@@ -20,7 +20,7 @@ logger = logging.getLogger(__name__)
 T = TypeVar("T")
 
 
-_DOWNLOAD_CHUNK_SIZE = Vec3Int(512, 512, 512)
+_DOWNLOAD_CHUNK_SHAPE = Vec3Int(512, 512, 512)
 
 
 def download_dataset(
@@ -54,8 +54,13 @@ def download_dataset(
         logger.warning(f"{actual_path} already exists, skipping download.")
         return Dataset.open(actual_path)
 
-    voxel_size = cast(Tuple[float, float, float], tuple(parsed.data_source.scale))
     data_layers = parsed.data_source.data_layers
+    scale = parsed.data_source.scale
+    if isinstance(data_layers, Unset) or isinstance(scale, Unset):
+        raise RuntimeError(
+            f"Could not download dataset {dataset_name}: {parsed.data_source.additional_properties.get('status', 'Unknown error.')}"
+        )
+    voxel_size = cast(Tuple[float, float, float], tuple(scale))
     dataset = Dataset(
         actual_path, name=parsed.name, voxel_size=voxel_size, exist_ok=exist_ok
     )
@@ -109,15 +114,15 @@ def download_dataset(
             mag_view = layer.get_or_add_mag(
                 mag,
                 compress=True,
-                chunk_size=Vec3Int.full(32),
-                chunks_per_shard=_DOWNLOAD_CHUNK_SIZE // 32,
+                chunk_shape=Vec3Int.full(32),
+                chunks_per_shard=_DOWNLOAD_CHUNK_SHAPE // 32,
             )
             aligned_bbox = layer.bounding_box.align_with_mag(mag, ceil=True)
-            download_chunk_size_in_mag = _DOWNLOAD_CHUNK_SIZE * mag.to_vec3_int()
+            download_chunk_shape_in_mag = _DOWNLOAD_CHUNK_SHAPE * mag.to_vec3_int()
             for chunk in track(
                 list(
                     aligned_bbox.chunk(
-                        download_chunk_size_in_mag, download_chunk_size_in_mag
+                        download_chunk_shape_in_mag, download_chunk_shape_in_mag
                     )
                 ),
                 description=f"Downloading layer={layer.name} mag={mag}",
@@ -127,7 +132,7 @@ def download_dataset(
                     organization_name=organization_id,
                     data_set_name=dataset_name,
                     data_layer_name=layer_name,
-                    resolution=mag.max_dim_log2,
+                    mag=mag.to_long_layer_name(),
                     client=datastore_client,
                     token=optional_datastore_token,
                     x=chunk.topleft.x,
