@@ -9,14 +9,16 @@ from pathlib import Path
 from typing import TYPE_CHECKING, Dict, List, Optional, Tuple, Union
 
 import numpy as np
+from cluster_tools import Executor
 from upath import UPath
 
 from webknossos.dataset.sampling_modes import SamplingModes
+from webknossos.dataset.view import _copy_job
 from webknossos.geometry import BoundingBox, Mag, Vec3Int, Vec3IntLike
 
 from ._array import ArrayException, BaseArray, DataFormat
 from ._downsampling_utils import (
-    calculate_default_max_mag,
+    calculate_default_coarsest_mag,
     calculate_mags_to_downsample,
     calculate_mags_to_upsample,
     determine_buffer_shape,
@@ -46,12 +48,17 @@ from ..utils import (
     warn_deprecated,
 )
 from .defaults import (
-    DEFAULT_CHUNK_SIZE,
+    DEFAULT_CHUNK_SHAPE,
     DEFAULT_CHUNKS_PER_SHARD,
     DEFAULT_CHUNKS_PER_SHARD_ZARR,
 )
 from .mag_view import MagView, _find_mag_path_on_disk
 
+try:
+    from numpy.typing import DTypeLike
+except ImportError:
+    DTypeLike = Union[str, np.dtype, type]  # type: ignore[misc]
+
 
 def _is_int(s: str) -> bool:
     try:
@@ -62,7 +69,7 @@ def _is_int(s: str) -> bool:
 
 
 def _convert_dtypes(
-    dtype: Union[str, np.dtype],
+    dtype: DTypeLike,
     num_channels: int,
     dtype_per_layer_to_dtype_per_channel: bool,
 ) -> str:
@@ -79,9 +86,7 @@ def _convert_dtypes(
     return "".join(converted_dtype_parts)
 
 
-def _normalize_dtype_per_channel(
-    dtype_per_channel: Union[str, np.dtype, type]
-) -> np.dtype:
+def _normalize_dtype_per_channel(dtype_per_channel: DTypeLike) -> np.dtype:
     try:
         return np.dtype(dtype_per_channel)
     except TypeError as e:
@@ -90,9 +95,7 @@ def _normalize_dtype_per_channel(
         ) from e
 
 
-def _normalize_dtype_per_layer(
-    dtype_per_layer: Union[str, np.dtype, type]
-) -> Union[str, np.dtype]:
+def _normalize_dtype_per_layer(dtype_per_layer: DTypeLike) -> DTypeLike:
     try:
         dtype_per_layer = str(np.dtype(dtype_per_layer))
     except Exception:
@@ -101,7 +104,7 @@ def _normalize_dtype_per_layer(
 
 
 def _dtype_per_layer_to_dtype_per_channel(
-    dtype_per_layer: Union[str, np.dtype], num_channels: int
+    dtype_per_layer: DTypeLike, num_channels: int
 ) -> np.dtype:
     try:
         return np.dtype(
@@ -116,7 +119,7 @@ def _dtype_per_layer_to_dtype_per_channel(
 
 
 def _dtype_per_channel_to_dtype_per_layer(
-    dtype_per_channel: Union[str, np.dtype], num_channels: int
+    dtype_per_channel: DTypeLike, num_channels: int
 ) -> str:
     return _convert_dtypes(
         np.dtype(dtype_per_channel),
@@ -126,7 +129,7 @@ def _dtype_per_channel_to_dtype_per_layer(
 
 
 def _dtype_per_channel_to_element_class(
-    dtype_per_channel: Union[str, np.dtype], num_channels: int
+    dtype_per_channel: DTypeLike, num_channels: int
 ) -> str:
     dtype_per_layer = _dtype_per_channel_to_dtype_per_layer(
         dtype_per_channel, num_channels
@@ -146,16 +149,21 @@ def _element_class_to_dtype_per_channel(
 
 
 def _get_sharding_parameters(
-    chunk_size: Optional[Union[Vec3IntLike, int]],
+    *,
+    chunk_shape: Optional[Union[Vec3IntLike, int]],
     chunks_per_shard: Optional[Union[Vec3IntLike, int]],
-    block_len: Optional[int],
-    file_len: Optional[int],
+    chunk_size: Optional[Union[Vec3IntLike, int]],  # deprecated
+    block_len: Optional[int],  # deprecated
+    file_len: Optional[int],  # deprecated
 ) -> Tuple[Optional[Vec3Int], Optional[Vec3Int]]:
-    if chunk_size is not None:
-        chunk_size = Vec3Int.from_vec_or_int(chunk_size)
+    if chunk_shape is not None:
+        chunk_shape = Vec3Int.from_vec_or_int(chunk_shape)
+    elif chunk_size is not None:
+        warn_deprecated("chunk_size", "chunk_shape")
+        chunk_shape = Vec3Int.from_vec_or_int(chunk_size)
     elif block_len is not None:
-        warn_deprecated("block_len", "chunk_size")
-        chunk_size = Vec3Int.full(block_len)
+        warn_deprecated("block_len", "chunk_shape")
+        chunk_shape = Vec3Int.full(block_len)
 
     if chunks_per_shard is not None:
         chunks_per_shard = Vec3Int.from_vec_or_int(chunks_per_shard)
@@ -163,7 +171,7 @@ def _get_sharding_parameters(
         warn_deprecated("file_len", "chunks_per_shard")
         chunks_per_shard = Vec3Int.full(file_len)
 
-    return (chunk_size, chunks_per_shard)
+    return (chunk_shape, chunks_per_shard)
 
 
 class Layer:
@@ -337,17 +345,19 @@ class Layer:
     def add_mag(
         self,
         mag: Union[int, str, list, tuple, np.ndarray, Mag],
-        chunk_size: Optional[Union[Vec3IntLike, int]] = None,  # DEFAULT_CHUNK_SIZE,
+        chunk_shape: Optional[Union[Vec3IntLike, int]] = None,  # DEFAULT_CHUNK_SHAPE,
         chunks_per_shard: Optional[
             Union[int, Vec3IntLike]
         ] = None,  # DEFAULT_CHUNKS_PER_SHARD,
         compress: bool = False,
+        *,
+        chunk_size: Optional[Union[Vec3IntLike, int]] = None,  # deprecated
         block_len: Optional[int] = None,  # deprecated
         file_len: Optional[int] = None,  # deprecated
     ) -> MagView:
         """
         Creates a new mag called and adds it to the layer.
-        The parameter `chunk_size`, `chunks_per_shard` and `compress` can be
+        The parameter `chunk_shape`, `chunks_per_shard` and `compress` can be
         specified to adjust how the data is stored on disk.
         Note that writing compressed data which is not aligned with the blocks on disk may result in
         diminished performance, as full blocks will automatically be read to pad the write actions. Alternatively,
@@ -362,24 +372,25 @@ class Layer:
         mag = Mag(mag)
         compression_mode = compress
 
-        chunk_size, chunks_per_shard = _get_sharding_parameters(
-            chunk_size=chunk_size,
+        chunk_shape, chunks_per_shard = _get_sharding_parameters(
+            chunk_shape=chunk_shape,
             chunks_per_shard=chunks_per_shard,
+            chunk_size=chunk_size,
             block_len=block_len,
             file_len=file_len,
         )
-        if chunk_size is None:
-            chunk_size = DEFAULT_CHUNK_SIZE
+        if chunk_shape is None:
+            chunk_shape = DEFAULT_CHUNK_SHAPE
         if chunks_per_shard is None:
             if self.data_format == DataFormat.Zarr:
                 chunks_per_shard = DEFAULT_CHUNKS_PER_SHARD_ZARR
             else:
                 chunks_per_shard = DEFAULT_CHUNKS_PER_SHARD
 
-        if chunk_size not in (Vec3Int.full(32), Vec3Int.full(64)):
+        if chunk_shape not in (Vec3Int.full(32), Vec3Int.full(64)):
             warnings.warn(
-                "[INFO] `chunk_size` of `32, 32, 32` or `64, 64, 64` is recommended for optimal "
-                + f"performance in webKnossos. Got {chunk_size}."
+                "[INFO] `chunk_shape` of `32, 32, 32` or `64, 64, 64` is recommended for optimal "
+                + f"performance in webKnossos. Got {chunk_shape}."
             )
 
         self._assert_mag_does_not_exist_yet(mag)
@@ -388,7 +399,7 @@ class Layer:
         mag_view = MagView(
             self,
             mag,
-            chunk_size=chunk_size,
+            chunk_shape=chunk_shape,
             chunks_per_shard=chunks_per_shard,
             compression_mode=compression_mode,
             create=True,
@@ -404,7 +415,7 @@ class Layer:
             MagViewProperties(
                 mag=Mag(mag_view.name),
                 cube_length=(
-                    mag_array_info.shard_size.x
+                    mag_array_info.shard_shape.x
                     if mag_array_info.data_format == DataFormat.WKW
                     else None
                 ),
@@ -436,7 +447,7 @@ class Layer:
             MagViewProperties(
                 mag=mag,
                 cube_length=(
-                    mag_array_info.shard_size.x
+                    mag_array_info.shard_shape.x
                     if mag_array_info.data_format == DataFormat.WKW
                     else None
                 ),
@@ -449,9 +460,11 @@ class Layer:
     def get_or_add_mag(
         self,
         mag: Union[int, str, list, tuple, np.ndarray, Mag],
-        chunk_size: Optional[Union[Vec3IntLike, int]] = None,
+        chunk_shape: Optional[Union[Vec3IntLike, int]] = None,
         chunks_per_shard: Optional[Union[Vec3IntLike, int]] = None,
         compress: Optional[bool] = None,
+        *,
+        chunk_size: Optional[Union[Vec3IntLike, int]] = None,  # deprecated
         block_len: Optional[int] = None,  # deprecated
         file_len: Optional[int] = None,  # deprecated
     ) -> MagView:
@@ -466,16 +479,17 @@ class Layer:
         mag = Mag(mag)
         compression_mode = compress
 
-        chunk_size, chunks_per_shard = _get_sharding_parameters(
-            chunk_size=chunk_size,
+        chunk_shape, chunks_per_shard = _get_sharding_parameters(
+            chunk_shape=chunk_shape,
             chunks_per_shard=chunks_per_shard,
+            chunk_size=chunk_size,
             block_len=block_len,
             file_len=file_len,
         )
 
         if mag in self._mags.keys():
             assert (
-                chunk_size is None or self._mags[mag].info.chunk_size == chunk_size
+                chunk_shape is None or self._mags[mag].info.chunk_shape == chunk_shape
             ), f"Cannot get_or_add_mag: The mag {mag} already exists, but the chunk sizes do not match"
             assert (
                 chunks_per_shard is None
@@ -489,7 +503,7 @@ class Layer:
         else:
             return self.add_mag(
                 mag,
-                chunk_size=chunk_size,
+                chunk_shape=chunk_shape,
                 chunks_per_shard=chunks_per_shard,
                 compress=compression_mode or False,
             )
@@ -518,67 +532,47 @@ class Layer:
         )
         rmtree(full_path)
 
-    def _add_foreign_mag(
+    def add_copy_mag(
         self,
         foreign_mag_view_or_path: Union[PathLike, str, MagView],
-        symlink: bool,
-        make_relative: bool,
         extend_layer_bounding_box: bool = True,
+        chunk_shape: Optional[Union[Vec3IntLike, int]] = None,
+        chunks_per_shard: Optional[Union[Vec3IntLike, int]] = None,
+        compress: Optional[bool] = None,
+        executor: Optional[Executor] = None,
     ) -> MagView:
         """
-        The foreign mag is (shallow) copied and the existing mag is added to the datasource-properties.json.
-        If extend_layer_bounding_box is true, the self.bounding_box will be extended
-        by the bounding box of the layer the foreign mag belongs to.
-        Symlinked mags can only be added to layers on local file systems.
+        Copies the data at `foreign_mag_view_or_path` which can belong to another dataset
+        to the current dataset. Additionally, the relevant information from the
+        `datasource-properties.json` of the other dataset are copied, too.
         """
         self.dataset._ensure_writable()
-
-        if isinstance(foreign_mag_view_or_path, MagView):
-            foreign_mag_view = foreign_mag_view_or_path
-        else:
-            # local import to prevent circular dependency
-            from .dataset import Dataset
-
-            foreign_mag_view_path = UPath(foreign_mag_view_or_path)
-            foreign_mag_view = (
-                Dataset.open(foreign_mag_view_path.parent.parent)
-                .get_layer(foreign_mag_view_path.parent.name)
-                .get_mag(foreign_mag_view_path.name)
-            )
-
+        foreign_mag_view = MagView._ensure_mag_view(foreign_mag_view_or_path)
         self._assert_mag_does_not_exist_yet(foreign_mag_view.mag)
 
-        if symlink:
-            assert is_fs_path(
-                self.path
-            ), f"Cannot create symlinks in remote layer {self.path}"
-            assert is_fs_path(
-                foreign_mag_view.path
-            ), f"Cannot create symlink to remote mag {foreign_mag_view.path}"
+        mag_view = self.add_mag(
+            mag=foreign_mag_view.mag,
+            chunk_shape=chunk_shape or foreign_mag_view._array_info.chunk_shape,
+            chunks_per_shard=chunks_per_shard
+            or foreign_mag_view._array_info.chunks_per_shard,
+            compress=compress
+            if compress is not None
+            else foreign_mag_view._array_info.compression_mode,
+        )
 
-            foreign_normalized_mag_path = (
-                Path(relpath(foreign_mag_view.path, self.path))
-                if make_relative
-                else foreign_mag_view.path.resolve()
-            )
-
-            (self.path / str(foreign_mag_view.mag)).symlink_to(
-                foreign_normalized_mag_path
-            )
-        else:
-            copytree(
-                foreign_mag_view.path,
-                self.path / str(foreign_mag_view.mag),
-            )
-
-        self.add_mag_for_existing_files(foreign_mag_view.mag)
         if extend_layer_bounding_box:
             self.bounding_box = self.bounding_box.extended_by(
                 foreign_mag_view.layer.bounding_box
             )
-        self.dataset._export_as_json()
 
-        return self._mags[foreign_mag_view.mag]
+        foreign_mag_view.for_zipped_chunks(
+            func_per_chunk=_copy_job,
+            target_view=mag_view,
+            executor=executor,
+            progress_desc=f"Copying mag {mag_view.mag.to_layer_name()} from {foreign_mag_view.layer} to {mag_view.layer}",
+        )
+
+        return mag_view
 
     def add_symlink_mag(
         self,
@@ -594,29 +588,60 @@ class Layer:
         If make_relative is True, the symlink is made relative to the current dataset path.
         Symlinked mags can only be added to layers on local file systems.
         """
-        return self._add_foreign_mag(
-            foreign_mag_view_or_path,
-            symlink=True,
-            make_relative=make_relative,
-            extend_layer_bounding_box=extend_layer_bounding_box,
+        self.dataset._ensure_writable()
+        foreign_mag_view = MagView._ensure_mag_view(foreign_mag_view_or_path)
+        self._assert_mag_does_not_exist_yet(foreign_mag_view.mag)
+
+        assert is_fs_path(
+            self.path
+        ), f"Cannot create symlinks in remote layer {self.path}"
+        assert is_fs_path(
+            foreign_mag_view.path
+        ), f"Cannot create symlink to remote mag {foreign_mag_view.path}"
+
+        foreign_normalized_mag_path = (
+            Path(relpath(foreign_mag_view.path, self.path))
+            if make_relative
+            else foreign_mag_view.path.resolve()
         )
 
-    def add_copy_mag(
+        (self.path / str(foreign_mag_view.mag)).symlink_to(foreign_normalized_mag_path)
+
+        mag = self.add_mag_for_existing_files(foreign_mag_view.mag)
+
+        if extend_layer_bounding_box:
+            self.bounding_box = self.bounding_box.extended_by(
+                foreign_mag_view.layer.bounding_box
+            )
+        return mag
+
+    def add_fs_copy_mag(
         self,
         foreign_mag_view_or_path: Union[PathLike, str, MagView],
         extend_layer_bounding_box: bool = True,
     ) -> MagView:
         """
-        Copies the data at `foreign_mag_view_or_path` which belongs to another dataset to the current dataset.
-        Additionally, the relevant information from the `datasource-properties.json` of the other dataset are copied too.
+        Copies the data at `foreign_mag_view_or_path` which belongs to another dataset to the current dataset via the filesystem.
+        Additionally, the relevant information from the `datasource-properties.json` of the other dataset are copied, too.
         """
-        return self._add_foreign_mag(
-            foreign_mag_view_or_path,
-            symlink=False,
-            make_relative=False,
-            extend_layer_bounding_box=extend_layer_bounding_box,
+        self.dataset._ensure_writable()
+        foreign_mag_view = MagView._ensure_mag_view(foreign_mag_view_or_path)
+        self._assert_mag_does_not_exist_yet(foreign_mag_view.mag)
+
+        copytree(
+            foreign_mag_view.path,
+            self.path / str(foreign_mag_view.mag),
         )
 
+        mag = self.add_mag_for_existing_files(foreign_mag_view.mag)
+
+        if extend_layer_bounding_box:
+            self.bounding_box = self.bounding_box.extended_by(
+                foreign_mag_view.layer.bounding_box
+            )
+
+        return mag
+
     def _create_dir_for_mag(
         self, mag: Union[int, str, list, tuple, np.ndarray, Mag]
     ) -> None:
@@ -634,21 +659,31 @@ class Layer:
                 )
             )
 
+    def _get_dataset_from_align_with_other_layers(
+        self, align_with_other_layers: Union[bool, "Dataset"]
+    ) -> Optional["Dataset"]:
+        if isinstance(align_with_other_layers, bool):
+            return self.dataset if align_with_other_layers else None
+        else:
+            return align_with_other_layers
+
     def downsample(
         self,
         from_mag: Optional[Mag] = None,
-        max_mag: Optional[Mag] = None,
+        coarsest_mag: Optional[Mag] = None,
         interpolation_mode: str = "default",
         compress: bool = True,
         sampling_mode: Union[str, SamplingModes] = SamplingModes.ANISOTROPIC,
+        align_with_other_layers: Union[bool, "Dataset"] = True,
         buffer_shape: Optional[Vec3Int] = None,
         force_sampling_scheme: bool = False,
-        args: Optional[Namespace] = None,
+        args: Optional[Namespace] = None,  # deprecated
         allow_overwrite: bool = False,
         only_setup_mags: bool = False,
+        executor: Optional[Executor] = None,
     ) -> None:
         """
-        Downsamples the data starting from `from_mag` until a magnification is `>= max(max_mag)`.
+        Downsamples the data starting from `from_mag` until a magnification is `>= max(coarsest_mag)`.
         There are three different `sampling_modes`:
         - 'anisotropic' - The next magnification is chosen so that the width, height and depth of a downsampled voxel assimilate. For example, if the z resolution is worse than the x/y resolution, z won't be downsampled in the first downsampling step(s). As a basis for this method, the voxel_size from the datasource-properties.json is used.
         - 'isotropic' - Each dimension is downsampled equally.
@@ -665,7 +700,7 @@ class Layer:
         assert "1" in self.mags.keys()
 
         layer.downsample(
-            max_mag=Mag(4),
+            coarsest_mag=Mag(4),
             sampling_mode=SamplingModes.ISOTROPIC
         )
 
@@ -683,8 +718,8 @@ class Layer:
             from_mag in self.mags.keys()
         ), f"Failed to downsample data. The from_mag ({from_mag.to_layer_name()}) does not exist."
 
-        if max_mag is None:
-            max_mag = calculate_default_max_mag(self.bounding_box.size)
+        if coarsest_mag is None:
+            coarsest_mag = calculate_default_coarsest_mag(self.bounding_box.size)
 
         sampling_mode = SamplingModes.parse(sampling_mode)
 
@@ -701,16 +736,21 @@ class Layer:
         elif sampling_mode == SamplingModes.ISOTROPIC:
             voxel_size = None
         elif sampling_mode == SamplingModes.CONSTANT_Z:
-            max_mag_with_fixed_z = max_mag.to_list()
-            max_mag_with_fixed_z[2] = from_mag.to_list()[2]
-            max_mag = Mag(max_mag_with_fixed_z)
+            coarsest_mag_with_fixed_z = coarsest_mag.to_list()
+            coarsest_mag_with_fixed_z[2] = from_mag.to_list()[2]
+            coarsest_mag = Mag(coarsest_mag_with_fixed_z)
             voxel_size = None
         else:
             raise AttributeError(
                 f"Downsampling failed: {sampling_mode} is not a valid SamplingMode ({SamplingModes.ANISOTROPIC}, {SamplingModes.ISOTROPIC}, {SamplingModes.CONSTANT_Z})"
             )
 
-        mags_to_downsample = calculate_mags_to_downsample(from_mag, max_mag, voxel_size)
+        dataset_to_align_with = self._get_dataset_from_align_with_other_layers(
+            align_with_other_layers
+        )
+        mags_to_downsample = calculate_mags_to_downsample(
+            from_mag, coarsest_mag, dataset_to_align_with, voxel_size
+        )
 
         if len(set([max(m.to_list()) for m in mags_to_downsample])) != len(
             mags_to_downsample
@@ -737,6 +777,7 @@ class Layer:
                 args=args,
                 allow_overwrite=allow_overwrite,
                 only_setup_mag=only_setup_mags,
+                executor=executor,
             )
 
     def downsample_mag(
@@ -746,9 +787,10 @@ class Layer:
         interpolation_mode: str = "default",
         compress: bool = True,
         buffer_shape: Optional[Vec3Int] = None,
-        args: Optional[Namespace] = None,
+        args: Optional[Namespace] = None,  # deprecated
         allow_overwrite: bool = False,
         only_setup_mag: bool = False,
+        executor: Optional[Executor] = None,
     ) -> None:
         """
         Performs a single downsampling step from `from_mag` to `target_mag`.
@@ -760,14 +802,23 @@ class Layer:
          - "bilinear"
          - "bicubic"
 
-        The `args` can contain information to distribute the computation.
         If allow_overwrite is True, an existing Mag may be overwritten.
 
         If only_setup_mag is True, the magnification is created, but left
         empty. This parameter can be used to prepare for parallel downsampling
         of multiple layers while avoiding parallel writes with outdated updates
         to the datasource-properties.json file.
+
+        `executor` can be passed to allow distrubuted computation, parallelizing
+        across chunks. `args` is deprecated.
         """
+
+        if args is not None:
+            warn_deprecated(
+                "args argument",
+                "executor (e.g. via webknossos.utils.get_executor_for_args(args))",
+            )
+
         assert (
             from_mag in self.mags.keys()
         ), f"Failed to downsample data. The from_mag ({from_mag.to_layer_name()}) does not exist."
@@ -811,7 +862,7 @@ class Layer:
         )
 
         # perform downsampling
-        with get_executor_for_args(args) as executor:
+        with get_executor_for_args(args, executor) as executor:
             if buffer_shape is None:
                 buffer_shape = determine_buffer_shape(prev_mag_view.info)
             func = named_partial(
@@ -834,7 +885,8 @@ class Layer:
         interpolation_mode: str = "default",
         compress: bool = True,
         buffer_shape: Optional[Vec3Int] = None,
-        args: Optional[Namespace] = None,
+        args: Optional[Namespace] = None,  # deprecated
+        executor: Optional[Executor] = None,
     ) -> None:
         """
         Use this method to recompute downsampled magnifications after mutating data in the
@@ -855,6 +907,7 @@ class Layer:
             buffer_shape=buffer_shape,
             args=args,
             allow_overwrite=True,
+            executor=executor,
         )
 
     def downsample_mag_list(
@@ -864,9 +917,10 @@ class Layer:
         interpolation_mode: str = "default",
         compress: bool = True,
         buffer_shape: Optional[Vec3Int] = None,
-        args: Optional[Namespace] = None,
+        args: Optional[Namespace] = None,  # deprecated
         allow_overwrite: bool = False,
         only_setup_mags: bool = False,
+        executor: Optional[Executor] = None,
     ) -> None:
         """
         Downsamples the data starting at `from_mag` to each magnification in `target_mags` iteratively.
@@ -899,6 +953,7 @@ class Layer:
                 args=args,
                 allow_overwrite=allow_overwrite,
                 only_setup_mag=only_setup_mags,
+                executor=executor,
             )
             source_mag = target_mag
 
@@ -908,9 +963,11 @@ class Layer:
         finest_mag: Mag = Mag(1),
         compress: bool = False,
         sampling_mode: Union[str, SamplingModes] = SamplingModes.ANISOTROPIC,
+        align_with_other_layers: Union[bool, "Dataset"] = True,
         buffer_shape: Optional[Vec3Int] = None,
         buffer_edge_len: Optional[int] = None,
-        args: Optional[Namespace] = None,
+        args: Optional[Namespace] = None,  # deprecated
+        executor: Optional[Executor] = None,
         *,
         min_mag: Optional[Mag] = None,
     ) -> None:
@@ -923,6 +980,13 @@ class Layer:
 
         `min_mag` is deprecated, please use `finest_mag` instead.
         """
+
+        if args is not None:
+            warn_deprecated(
+                "args argument",
+                "executor (e.g. via webknossos.utils.get_executor_for_args(args))",
+            )
+
         assert (
             from_mag in self.mags.keys()
         ), f"Failed to upsample data. The from_mag ({from_mag.to_layer_name()}) does not exist."
@@ -954,7 +1018,12 @@ class Layer:
         if buffer_shape is None and buffer_edge_len is not None:
             buffer_shape = Vec3Int.full(buffer_edge_len)
 
-        mags_to_upsample = calculate_mags_to_upsample(from_mag, finest_mag, voxel_size)
+        dataset_to_align_with = self._get_dataset_from_align_with_other_layers(
+            align_with_other_layers
+        )
+        mags_to_upsample = calculate_mags_to_upsample(
+            from_mag, finest_mag, dataset_to_align_with, voxel_size
+        )
 
         for prev_mag, target_mag in zip(
             [from_mag] + mags_to_upsample[:-1], mags_to_upsample
@@ -977,7 +1046,7 @@ class Layer:
             target_view = target_mag_view.get_view()
 
             # perform upsampling
-            with get_executor_for_args(args) as executor:
+            with get_executor_for_args(args, executor) as actual_executor:
 
                 if buffer_shape is None:
                     buffer_shape = determine_buffer_shape(prev_mag_view.info)
@@ -990,7 +1059,7 @@ class Layer:
                     # this view is restricted to the bounding box specified in the properties
                     func,
                     target_view=target_view,
-                    executor=executor,
+                    executor=actual_executor,
                     progress_desc=f"Upsampling from Mag {prev_mag} to Mag {target_mag}",
                 )
 
@@ -1009,14 +1078,14 @@ class Layer:
             self._mags[mag] = MagView(
                 self,
                 mag,
-                info.chunk_size,
+                info.chunk_shape,
                 info.chunks_per_shard,
                 info.compression_mode,
             )
             self._mags[mag]._read_only = self._dataset.read_only
-        except ArrayException as e:
-            logging.error(
-                f"Failed to setup magnification {mag_name}, which is specified in the datasource-properties.json. See {e}"
+        except ArrayException:
+            logging.exception(
+                f"Failed to setup magnification {mag_name}, which is specified in the datasource-properties.json:"
             )
 
     def _initialize_mag_from_other_mag(
@@ -1024,33 +1093,41 @@ class Layer:
     ) -> MagView:
         return self.add_mag(
             new_mag_name,
-            chunk_size=other_mag.info.chunk_size,
+            chunk_shape=other_mag.info.chunk_shape,
             chunks_per_shard=other_mag.info.chunks_per_shard,
             compress=compress,
         )
 
     def __repr__(self) -> str:
-        return repr(
-            "Layer(%s, dtype_per_channel=%s, num_channels=%s)"
-            % (self.name, self.dtype_per_channel, self.num_channels)
-        )
+        return f"Layer({repr(self.name)}, dtype_per_channel={self.dtype_per_channel}, num_channels={self.num_channels})"
 
     def _get_largest_segment_id_maybe(self) -> Optional[int]:
         return None
 
+    @classmethod
+    def _ensure_layer(cls, layer: Union[str, PathLike, "Layer"]) -> "Layer":
+        if isinstance(layer, Layer):
+            return layer
+        else:
+            # local import to prevent circular dependency
+            from .dataset import Dataset
+
+            layer_path = UPath(layer)
+            return Dataset.open(layer_path.parent).get_layer(layer_path.name)
+
 
 class SegmentationLayer(Layer):
 
     _properties: SegmentationLayerProperties
 
     @property
-    def largest_segment_id(self) -> int:
+    def largest_segment_id(self) -> Optional[int]:
         return self._properties.largest_segment_id
 
     @largest_segment_id.setter
-    def largest_segment_id(self, largest_segment_id: int) -> None:
+    def largest_segment_id(self, largest_segment_id: Optional[int]) -> None:
         self.dataset._ensure_writable()
-        if type(largest_segment_id) != int:
+        if largest_segment_id is not None and type(largest_segment_id) != int:
             assert largest_segment_id == int(
                 largest_segment_id
             ), f"A non-integer value was passed for largest_segment_id ({largest_segment_id})."
